<?php
class DBLayer
{
    protected $_db;
    protected $_entity;
    protected $_table;
    protected $_pk;

    /**
     * @param $entity
     * @param $table
     * @param $pk
     */
    public function __construct($entity, $table, $pk)
    {
        $this->_entity = $entity;
        $this->_table = $table;
        $this->_pk = $pk;
        $this->_db = JFactory::getDBO();
    }

    /**
     * @param array $filter
     * @return bool
     */
    public function exists($filter = array())
    {
        $row = $this->findOne($filter);
        if (is_string($row))
            trigger_error($row, E_USER_ERROR);
        return !is_null($row);
    }

    /**
     * @param array $filter
     * @return null
     */
    public function findOne($filter = array())
    {
        $filter['limit'] = 1;
        $list = $this->find($filter);
        if (is_string($list))
            trigger_error($list, E_USER_ERROR);
        if (0 == count($list)) {
            $null = null;
            return $null;
        }
        return $list[0];
    }

    /**
     * @param array $filter
     * @return array
     */
    public function find($filter = array())
    {
        $result = $this->_loadObjects();
        return $result;
    }

    /**
     * @param $id
     * @return mixed
     */
    public function fetch($id)
    {
        $row = JTable::getInstance($this->_entity);
        $row->load($id);
        return $row;
    }

    /**
     * @param $id
     * @return null
     */
    public function delete($id)
    {
        $row = $this->fetch($id);
        if (!$row->delete())
            trigger_error($row->getError(), E_USER_ERROR);
        return null;
    }

    /**
     * @param $row
     * @return null
     */
    public function save($row)
    {
        if (!$row->check())
            trigger_error($row->getError(), E_USER_ERROR);
        if (!$row->store())
            trigger_error($row->getError(), E_USER_ERROR);
        if (!$row->checkin())
            trigger_error($row->getError(), E_USER_ERROR);
        return null;
    }

    /**
     * @return mixed
     */
    protected function _create()
    {
        $result = JTable::getInstance($this->_entity);
        return $result;
    }

    /**
     * @param array $where
     * @param int $limit
     * @param string $orderby
     * @return array
     */
    protected function _loadObjects($where = array(), $limit = 0, $orderby = '')
    {
        if (empty($orderby))
            $orderby = $this->_pk;
        $query = 'SELECT * FROM #__' . $this->_table
            . (count($where) ? ' WHERE ' . implode(' AND ', $where) : '')
            . ' ORDER BY ' . $orderby;
        $this->_db->setQuery($query, 0, $limit);

        $rows = $this->_db->loadAssocList();
        if ($this->_db->getErrorNum())
            trigger_error($this->_db->stderr(), E_USER_ERROR);
        $result = array();
        for ($i = 0; $i < count($rows); $i++) {
            $result[$i] = JTable::getInstance($this->_entity);
            $result[$i]->bind($rows[$i]);
        }
        return $result;
    }
}